/*
 * Copyright (c) 2018 Includable.
 * Created by Thomas Schoffelen.
 */

'use strict'

const systemPath = require('path')
const chokidar = require('chokidar')

const openBrowser = require('@includable/open-browser')

const util = require('../util')
const output = require('../output')
const helpers = require('../run-helpers')
const Container = require('../containers/Container')
const shouldIgnore = require('../plugins/Ignore')

function startWithContainer (containerToken, envr, silent) {
  output.notice('Running on ' + containerToken + '.' + (global.containerEndpoint
    .replace('https://api.', '')
    .replace('http://api.', '')
    .replace('/', '')) + ' as ' + envr)

  // Start web socket connection
  helpers.startWebSocket(containerToken)

  // Pre-run scripts
  helpers.startChildProcess('pre-run', function () {
    // Upload initial zip file
    helpers.createInitialZip('.', containerToken, envr, function () {
      chokidar.watch('.', {
        ignored: /[/\\]\./,
        ignorePermissionErrors: true,
        ignoreInitial: true,
        followSymlinks: true,
        atomic: 200,
        awaitWriteFinish: {
          stabilityThreshold: 90,
          pollInterval: 30
        }
      }).on('all', function (event, origPath) {
        const path = origPath.split(systemPath.sep).join('/')
        if (path === '.' || path === '..' || path.match(/(\.DS_Store$|^vendor\/|(^|\/)node_modules\/|(^|\/)\.phpstorm\.meta\.php$|^\.(inc|git|idea)(ignore|\/|$))/)) {
          return
        }

        if (shouldIgnore(path)) {
          return
        }

        if (event === 'addDir' && path === 'vendor') {
          output.warn('Ignoring directory \'vendor\'')
          return
        }

        setTimeout(function () {
          try {
            util.statOrNothing(origPath, event, function (err, ok) {
              if (err || !ok) {
                return
              }
              const actionString = 'File ' + event + ' ' + path
              const stopSpinner = output.wait(helpers.logSyncMessage(actionString, true))
              helpers.uploadFile(containerToken, event, path, function (result) {
                if (!result || !result.length) {
                  stopSpinner(true)
                } else if (result.indexOf('Error:') === 0) {
                  stopSpinner(result)
                } else {
                  stopSpinner()
                  helpers.logSyncMessage(result)
                }
              })
            })
          } catch (e) {
            output.err(e.message)
          }
        }, event === 'unlink' ? 500 : 50)
      })

      helpers.uploadFile(containerToken, 'post', '.')

      helpers.startChildProcess('run')

      if (!silent) {
        const url = global.containerEndpoint + containerToken + '/login/' + envr
        output.debug('Opening URL:', url)
        openBrowser(url)
      }

      util.downloadHelper()
    })
  })
}

function startWithConfig (environment, options, containerName) {
  Container.getConfig(containerName, function (err, config) {
    // Use existing container
    if (!err && config) {
      output.debug('Found existing container config for container', config.containerToken)
      startWithContainer(config.containerToken, environment, options.silent)
      return
    }

    // Create a new container
    output.debug('Requesting to create a new container')
    Container.create(options, function (err, result) {
      if (err) {
        output.err(err)
        return
      }

      output.debug('Starting with a new container with token', result.container_token)
      startWithContainer(result.container_token, environment, options.silent)
    })
  })
}

/**
 * @param {Object} options
 */
module.exports = function (options) {
  // Enable beta options
  if (options.beta) {
    output.warn('enabled usage of the beta API')
    process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0'
    global.containerEndpoint = 'http://api.local.includable.com/'
    global.apiStability = 'beta'
  }

  // Enable verbose option
  if (options.verbose) {
    global.verbose = true
  }

  // Determine environment
  const environment = options.environment && typeof options.environment === 'string'
    ? options.environment : 'user'

  // Use container ID specified by user in command
  if (options.container && typeof options.container === 'string') {
    startWithContainer(options.container, environment, options.silent)
    return
  }

  // Check if we need to upgrade the project
  Container.performUpgrade()

  // Start with local config files in '.inc'
  startWithConfig(environment, options, options.beta ? 'beta' : 'default')
}
